package com.sd.business.flow;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.fjhx.flow.core.FlowDelegate;
import com.fjhx.flow.enums.FlowStatusEnum;
import com.ruoyi.common.core.domain.BaseIdPo;
import com.ruoyi.common.core.domain.BasePo;
import com.ruoyi.common.utils.SecurityUtils;
import com.sd.business.entity.apply.po.ApplyBuy;
import com.sd.business.entity.apply.po.ApplyBuyBom;
import com.sd.business.entity.purchase.dto.PurchaseDto;
import com.sd.business.entity.purchase.enums.PurchaseEnums;
import com.sd.business.entity.purchase.po.Purchase;
import com.sd.business.entity.purchase.po.PurchaseBom;
import com.sd.business.service.apply.ApplyBuyBomService;
import com.sd.business.service.apply.ApplyBuyService;
import com.sd.business.service.purchase.PurchaseBomService;
import com.sd.business.service.purchase.PurchaseService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * 发起流程
 */
@Service
public class PurchaseFlow  extends FlowDelegate {

    @Autowired
    private PurchaseService purchaseService;

    @Autowired
    private PurchaseBomService purchaseBomService;

    @Autowired
    private ApplyBuyService applyBuyService;

    @Autowired
    private ApplyBuyBomService  applyBuyBomService;

    @Override
    public String getFlowKey() {
        return "purchase";
    }

    @Override
    public Long start(Long flowId, JSONObject submitData) {

        PurchaseDto purchase = submitData.toJavaObject(PurchaseDto.class);
        purchase.setFlowStatus(PurchaseEnums.IN_PROGRESS.getKey());
        purchase.setFlowId(flowId);

        if (purchase.getId() == null) {
            purchaseService.add(purchase);
        } else {
            purchaseService.edit(purchase);
        }

        return purchase.getId();
    }

    @Override
    public void end(Long flowId, Long businessId, JSONObject submitData) {
        Purchase purchase = purchaseService.getById(businessId);
        purchase.setFlowStatus(PurchaseEnums.PASSED.getKey());
        purchase.setFlowId(flowId);
        // 修改采购暂存表的数据
        purchaseService.updateById(purchase);

        // 修改申购表的状态为已申购
        ApplyBuy applyBuy = new ApplyBuy();
        applyBuy.setId(purchase.getApplyBuyId());
        applyBuy.setFlowStatus(4);
        applyBuyService.updateById(applyBuy);

        // 查询采购合同bom表的信息
        List<PurchaseBom> purchaseBoms = purchaseBomService.list(Wrappers.<PurchaseBom>lambdaQuery().eq(PurchaseBom::getPurchaseId, businessId));

        List<ApplyBuyBom> applyBuyBomList = new ArrayList<>();

        // 修改申购bom表的采购数量
        for (PurchaseBom purchaseBom : purchaseBoms) {
            ApplyBuyBom applyBuyBom = new ApplyBuyBom();
            applyBuyBom.setPurchaseQuantity(purchaseBom.getPurchaseQuantity());
            applyBuyBom.setId(purchaseBom.getApplyBuyBomId());
            applyBuyBomList.add(applyBuyBom);
        }
        applyBuyBomService.updateBatchById(applyBuyBomList);

    }

    @Override
    public void returnToOriginator(Long flowId, Long businessId, FlowStatusEnum flowStatus) {
        purchaseService.update(q -> q
                .eq(BaseIdPo::getId, businessId)
                .set(Purchase::getFlowStatus, flowStatus.getKey())
                .set(BasePo::getUpdateTime, new Date())
                .set(BasePo::getUpdateUser, SecurityUtils.getUserId())
        );
    }

    @Override
    public void relaunch(Long flowId, Long businessId, FlowStatusEnum flowStatus, JSONObject submitData) {
        PurchaseDto purchase = submitData.toJavaObject(PurchaseDto.class);
        purchase.setFlowStatus(FlowStatusEnum.IN_PROGRESS.getKey());
        purchaseService.edit(purchase);
    }

    @Override
    public void reject(Long flowId, Long businessId, FlowStatusEnum flowStatus) {
        purchaseService.update(q -> q
                .eq(BaseIdPo::getId, businessId)
                .set(Purchase::getFlowStatus, flowStatus.getKey())
                .set(BasePo::getUpdateTime, new Date())
                .set(BasePo::getUpdateUser, SecurityUtils.getUserId())
        );
    }

}
